/*
    Tint Filter for VirtualDub -- Apply a tint 
	with selectable color and alpha blending.
	Copyright (C) 2007 Donald A. Graft

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Donald Graft
	neuron2 AT comcast.net.
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <crtdbg.h>
#include <math.h>
#include <stdlib.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

///////////////////////////////////////////////////////////////////////////

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////

typedef struct MyFilterData {
	IFilterPreview		*ifp;
	int					strength;
	int                 xr, xg, xb;
} MyFilterData;

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    _snprintf(buf, buflen, "Config(%d, %d, %d, %d)",
		mfd->strength,
		mfd->xr,
		mfd->xg,
		mfd->xb);

	return true;
}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc)
{
	void CountLogoBMPs(MyFilterData *mfd);
	FilterActivation *fa = (FilterActivation *)lpVoid;
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->strength		= argv[0].asInt();
	mfd->xr				= argv[1].asInt();
	mfd->xg				= argv[2].asInt();
	mfd->xb				= argv[3].asInt();
}

ScriptFunctionDef func_defs[]={
	{ (ScriptFunctionPtr)ScriptConfig, "Config", "0iiii" },
	{ NULL },
};

CScriptObject script_obj={
	NULL, func_defs
};

struct FilterDefinition filterDef_Tint = {

	NULL, NULL, NULL,		// next, prev, module
	"tint (1.0)",	        // name
	"Apply a tint color with alpha blending.",
							// desc
	"Donald Graft", 		// maker
	NULL,					// private_data
	sizeof(MyFilterData),	// inst_data_size

	InitProc,				// initProc
	NULL,					// deinitProc
	RunProc,				// runProc
	ParamProc,				// paramProc
	ConfigProc, 			// configProc
	StringProc, 			// stringProc
	StartProc,				// startProc
	EndProc,				// endProc

	&script_obj,			// script_obj
	FssProc,				// fssProc

};

int StartProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	return 0;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	return 0;
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	const PixDim	width = fa->src.w;
	const PixDim	height = fa->src.h;
	Pixel32 *src;
	int x, y;
	int r, g, b;
	int rb, gb, bb;
	int XR = mfd->xr;
	int XG = mfd->xg;
	int XB = mfd->xb;
    double ALPHA;

	// Now render the bitmap onto the current frame.
	src	= (Pixel *)((char *)fa->src.data);
	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width; x++)
		{
			r = ((src[x] & 0xff0000) >> 16);
			g = ((src[x] & 0xff00) >> 8);
			b = (src[x] & 0xff);
            ALPHA = (mfd->strength / 1000.0);
			bb = (int) (ALPHA * XB + (1 - ALPHA) * b);			 
			gb = (int) (ALPHA * XG + (1 - ALPHA) * g);			 
			rb = (int) (ALPHA * XR + (1 - ALPHA) * r);			 
			src[x] = ((rb << 16) | (gb << 8) | (bb));
		}
		src	= (Pixel *)((char *)src + fa->src.pitch);
	}
	return 0;
}

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_Tint;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_Tint = ff->addFilter(fm, &filterDef_Tint, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_Tint);
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->strength = 0;
	mfd->xr = 0;
	mfd->xg = 0;
	mfd->xb = 0;
	return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	switch(msg) {
		case WM_INITDIALOG:
			SetWindowLong(hdlg, DWL_USER, lParam);
			mfd = (MyFilterData *)lParam;

			EnableWindow(GetDlgItem(hdlg, IDC_RED), TRUE);
			EnableWindow(GetDlgItem(hdlg, IDC_BLUE), TRUE);
			EnableWindow(GetDlgItem(hdlg, IDC_GREEN), TRUE);

			SendMessage(GetDlgItem(hdlg, IDC_SLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 1000));
			SendMessage(GetDlgItem(hdlg, IDC_SLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->strength);
			SetDlgItemInt(hdlg, IDC_STRENGTH, mfd->strength, FALSE);
			
			SetDlgItemInt(hdlg, IDC_RED, mfd->xr, FALSE);
			SetDlgItemInt(hdlg, IDC_GREEN, mfd->xg, FALSE);
			SetDlgItemInt(hdlg, IDC_BLUE, mfd->xb, FALSE);
			mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
			return TRUE;

		case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER))
			{
				int x = SendMessage(GetDlgItem(hdlg, IDC_SLIDER), TBM_GETPOS, 0, 0);
				if (x != mfd->strength)
				{
					mfd->strength = x;
					SetDlgItemInt(hdlg, IDC_STRENGTH, mfd->strength, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
            break;

		case WM_COMMAND:
			switch(LOWORD(wParam)) {

			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				break;

			case IDOK:
				EndDialog(hdlg, 0);
				return TRUE;
			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;

				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\Tint.html");
				ShellExecute(hdlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				return TRUE;
				}

			case IDC_RED:
				mfd->xr = GetDlgItemInt(hdlg, IDC_RED, 0, FALSE);
				if (mfd->xr > 255)
				{
					SetDlgItemInt(hdlg, IDC_RED, 255, FALSE);
					mfd->xr = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_GREEN:
				mfd->xg = GetDlgItemInt(hdlg, IDC_GREEN, 0, FALSE);
				if (mfd->xg > 255)
				{
					SetDlgItemInt(hdlg, IDC_GREEN, 255, FALSE);
					mfd->xg = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_BLUE:
				mfd->xb = GetDlgItemInt(hdlg, IDC_BLUE, 0, FALSE);
				if (mfd->xb > 255)
				{
					SetDlgItemInt(hdlg, IDC_BLUE, 255, FALSE);
					mfd->xb = 255;
				}
				mfd->ifp->RedoFrame();
				break;

            case IDC_PLUS:
				if (mfd->strength < 1000)
				{
					mfd->strength += 1;
					SetDlgItemInt(hdlg, IDC_STRENGTH, mfd->strength, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_SLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->strength);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_MINUS:
				if (mfd->strength > 0)
				{
					mfd->strength -= 1;
					SetDlgItemInt(hdlg, IDC_STRENGTH, mfd->strength, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_SLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->strength);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDCANCEL:
				EndDialog(hdlg, 1);
				return TRUE;
			}
			break;
	}

	return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd_old = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

	if (DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER),
		hwnd, ConfigDlgProc, (LPARAM)mfd))
	{
		*mfd = mfd_old;
		ret = TRUE;
	}
    else
	{
		ret = FALSE;
	}
	return(ret);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
}

long ParamProc(FilterActivation *fa, const FilterFunctions *ff)
{
	fa->dst.offset	= fa->src.offset;
	fa->dst.modulo	= fa->src.modulo;
	fa->dst.pitch	= fa->src.pitch;
	return 0;
}
